@@ -63,9 +63,9 @@ fetchNASISLabData <- function(SS = TRUE, dsn = NULL) {
 	hzidname(h) <- 'labphiid'
 
 	# 7. save and mention bad pedons
-	assign('bad.labpedon.ids', value = bad.pedon.ids, envir = soilDB.env)
+	assign('bad.labpedon.ids', value = bad.pedon.ids, envir = get_soilDB_env())
 	if (length(bad.pedon.ids) > 0)
-		message("horizon errors detected, use `get('bad.labpedon.ids', envir=soilDB.env)` for a list of pedon IDs")
+		message("horizon errors detected, use `get('bad.labpedon.ids', envir=get_soilDB_env())` for a list of pedon IDs")
 
 	# done
 	return(h)
